<?php
/*
Plugin Name: WooCommerce - INITPRO Касса
Description: Фискализация платежей с помощью сервиса INITPRO Касса для плагина WooCommerce
Plugin URI: http://wordpress.org/plugins/initprokassa/
Author: Initpro
Version: 1.0.2
Author URI: http://kassa.initpro.ru/
*/

use Initpro\KassaSdk\CalculationMethod;
use Initpro\KassaSdk\CalculationSubject;
use Initpro\KassaSdk\Client;
use Initpro\KassaSdk\GroupManager;
use Initpro\KassaSdk\Check;
use Initpro\KassaSdk\Payment;
use Initpro\KassaSdk\Position;
use Initpro\KassaSdk\Vat;
use Initpro\KassaSdk\Exception\SdkException;

final class InitproKassa
{

    public $version = '1.0.2';

    const DEFAULT_QUEUE_NAME = 'default';
    const DISCOUNT_NOT_AVAILABLE = 0;

    private static $_instance = null;

    public static function instance()
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function __construct()
    {
        $this->define('INITPROKASSA_ABSPATH', plugin_dir_path(__FILE__));
        $this->define('INITPROKASSA_ABSPATH_VIEWS', plugin_dir_path(__FILE__) . 'includes/views/');
        $this->define('INITPROKASSA_BASENAME', plugin_basename(__FILE__));

        $this->includes();
        $this->hooks();
        $this->wp_hooks();
        $this->wp_endpoints();
        $this->load_options();
        $this->init();
    }

    public function wp_hooks()
    {
        register_activation_hook(__FILE__, array('InitproKassa_Install', 'activation'));

        add_action('woocommerce_order_status_' . get_option('initprokassa_fiscalize_prepayment'), [$this, 'fiscalize']);
        add_action('woocommerce_order_status_' . get_option('initprokassa_fiscalize_full_payment'), [$this, 'fiscalize']);
        add_action('woocommerce_order_status_' . get_option('initprokassa_fiscalize_single_receipt_full_payment'), [$this, 'fiscalize']);
        add_action('woocommerce_order_status_' . get_option('initprokassa_fiscalize_refund'), [$this, 'fiscalize']);
    }

    public function wp_endpoints()
    {
        add_filter('query_vars', array($this, 'add_query_vars'), 0);
        add_action('init', array($this, 'add_endpoint'), 0);
        add_action('parse_request', array($this, 'handle_requests'), 0);
    }

    public function hooks()
    {
        add_action('initpro_kassa_action_success', array($this, 'action_success'));
        add_action('initpro_kassa_action_fail', array($this, 'action_fail'));
        add_action('initpro_kassa_report_create', array($this, 'report_create'), 10, 4);
        add_action('initpro_kassa_report_update', array($this, 'report_update'), 10, 3);
        add_action('wp_loaded', array($this, 'handle_action'));
    }

    public function includes()
    {
        require_once(INITPROKASSA_ABSPATH . 'includes/class-initprokassa-install.php');
        require_once(INITPROKASSA_ABSPATH . 'includes/class-initprokassa-report.php');
        require_once(INITPROKASSA_ABSPATH . 'includes/libs/initpro-kassa-php-sdk/autoload.php');

        if (is_admin()) {
            require_once(INITPROKASSA_ABSPATH . 'includes/class-initprokassa-admin.php');
            add_action('init', array('InitproKassa_Admin', 'init'));
        }
    }

    private function define($name, $value)
    {
        if (!defined($name)) {
            define($name, $value);
        }
    }

    public function load_options()
    {
        $this->server_url = get_option('initprokassa_server_url');
        $this->shop_id = get_option('initprokassa_shop_id');
        $this->secret_key = get_option('initprokassa_secret_key');
        $this->queue_id = get_option('initprokassa_queue_id');
    }

    public function init()
    {
        do_action('before_initprokassa_init');
        $this->client = new Client($this->shop_id, $this->secret_key);
        $this->client->setHost($this->server_url);
        $this->manager = new GroupManager($this->client);
        $this->manager->registerGroup(self::DEFAULT_QUEUE_NAME, $this->queue_id);
        $this->manager->setDefaultGroup(self::DEFAULT_QUEUE_NAME);
        $this->report = new InitproKassa_Report();
        do_action('initprokassa_init');
    }

    public function fiscalize($order_id)
    {
        $order = wc_get_order($order_id);
        if (!$order) return;

        $status = $order->get_status();

        if (!in_array(
            $order->get_payment_method(),
            json_decode(get_option("initprokassa_payment_methods"), true)
        )) return;

        $tax_system = intval(get_option('initprokassa_tax_system'));

        if ($status == get_option('initprokassa_fiscalize_refund')) {
            $refundStatus = Check::INTENT_SELL_RETURN;
            $type = 'refund';
        } else {
            $refundStatus = Check::INTENT_SELL;
            $type = 'sell';
        }

        $uniqid = uniqid(strval($order_id), true);

        $check = new Check($uniqid, $order->get_billing_email(), $refundStatus, $tax_system);

        $calculationSubject = get_option('initprokassa_calculation_subject');
        $calculationSubjectDelivery = get_option('initprokassa_calculation_subject_delivery');

        if (
            $status == get_option('initprokassa_fiscalize_refund')
            || $status == get_option('initprokassa_fiscalize_single_receipt_full_payment')
        ) {
            $calculationMethod = CalculationMethod::FULL_PAYMENT;
            $paymentType = Payment::TYPE_CARD;
        }

        if ($status == get_option('initprokassa_fiscalize_prepayment')) {
            $calculationMethod = CalculationMethod::PRE_PAYMENT_FULL;
            $paymentType = Payment::TYPE_CARD;
            $calculationSubject = CalculationSubject::PAYMENT;
            $calculationSubjectDelivery = CalculationSubject::PAYMENT;
        }

        if ($status == get_option('initprokassa_fiscalize_full_payment')) {
            $calculationMethod = CalculationMethod::FULL_PAYMENT;
            $paymentType = Payment::TYPE_PREPAYMENT;
        }

        if (sizeof($order->get_items()) > 0) {

            foreach ($order->get_items('line_item') as $item) {
                $position = new Position(
                    $item->get_name(),
                    $order->get_item_total($item, true, true),
                    $item->get_quantity(),
                    $order->get_line_total($item, true, true),
                    $order->get_line_subtotal($item, false, true) - $order->get_item_total($item, false, true),
                    new Vat(get_option('initprokassa_vat'))
                );
                $position->setCalculationMethod($calculationMethod);
                $position->setCalculationSubject($calculationSubject);

                $check->addPosition($position);
            }
            // shipping

            foreach ($order->get_items('shipping') as $item) {
                $position = new Position(
                    $item->get_name(),
                    $order->get_item_total($item, true, true),
                    $item->get_quantity(),
                    $order->get_line_total($item, true, true),
                    self::DISCOUNT_NOT_AVAILABLE,
                    new Vat(get_option('initprokassa_vat_delivery'))
                );
                $position->setCalculationMethod($calculationMethod);
                $position->setCalculationSubject($calculationSubjectDelivery);

                $check->addPosition($position);
            }
        }

        $check->addPayment(new Payment($paymentType, round($order->get_total(), 2)));

        $error_message = "";
        $response = null;
        try {
            $response = $this->manager->putCheck($check);
        } catch (SdkException $e) {
            $error_message = $e->getMessage();
        }
        do_action('initpro_kassa_report_create', $uniqid, $order_id, $type, $check->asArray(), $response, $error_message);
    }

    public function add_query_vars($vars)
    {
        $vars[] = 'initpro-kassa';
        return $vars;
    }

    public static function add_endpoint()
    {
        add_rewrite_endpoint('initpro-kassa', EP_ALL);
    }

    public function handle_requests()
    {
        global $wp;

        if (empty($wp->query_vars['initpro-kassa'])) {
            return;
        }

        $initpro_kassa_action = strtolower(wc_clean($wp->query_vars['initpro-kassa']));
        do_action('initpro_kassa_action_' . $initpro_kassa_action);
        die(-1);
    }

    public function handle_action()
    {
        global $wp;

        if ($_SERVER['REQUEST_URI'] == '/initpro-kassa-report' && $_SERVER['REQUEST_METHOD'] == 'POST') {
            if (!array_key_exists('HTTP_X_HMAC_SIGNATURE', $_SERVER)) {
                status_header(401);
                exit();
            }
            if ($_SERVER['REQUEST_METHOD'] != 'POST') {
                status_header(405);
                header('Allow: POST');
                exit();
            }

            if (empty($this->secret_key)) {
                error_log('Unable to handle request: initprokassa_secret_key is not defined');
                status_header(500);
            }

            $scheme = array_key_exists('HTTPS', $_SERVER) && strtolower($_SERVER['HTTPS']) !== 'off' ? 'https' : 'http';
            $url = sprintf('%s://%s%s', $scheme, $_SERVER['SERVER_NAME'], $_SERVER['REQUEST_URI']);
            $data = file_get_contents('php://input');
            $signature = hash_hmac('md5', $_SERVER['REQUEST_METHOD'] . $url . $data, $this->secret_key);

            if ($signature != $_SERVER['HTTP_X_HMAC_SIGNATURE']) {
                status_header(403);
                exit();
            }

            $data = json_decode($data, true);

            foreach (array('external_id', 'state') as $key) {
                if (!array_key_exists($key, $data)) {
                    status_header(422);
                    header('Content-Type: text/plain');
                    echo $key . " is required\n";
                    exit();
                }
            }

            do_action('initpro_kassa_report_update', $data['external_id'], $data['state'], $data);
        }
    }

    public function report_create($uniqid, $order_id, $type, $request_check_data, $response_data = "", $error = "")
    {
        $this->report->create($uniqid, $order_id, $type, $request_check_data, $response_data, $error);
    }

    public function report_update($uniqid, $state, $report_data)
    {
        $this->report->update($uniqid, $state, $report_data);
    }
}

function Initpro_Kassa()
{
    return InitproKassa::instance();
}

$GLOBALS['initprokassa'] = Initpro_Kassa();
